// Constant values for digit selection using the potentiometer
//   1 band for each digit POT_M
//   1 dead band between each digit POT_N
//   10M + 9N = 1024 counts
#define POT_M 75
#define POT_N 30

void EvalControls() {
   ButtonPressShort = false;
   ButtonPressLong = false;
    
  if (--DebounceTimer == 0) {
    DebounceTimer = DEBOUNCE_CNT;
  
    // Sample the pushbutton
    boolean curButton;
    if (digitalRead(MODE_BUTTON) == 1) {
      curButton = false;
    } else {
      curButton = true;
    }
    ButtonReleased = !curButton && ButtonPrev && ButtonDown;
    ButtonDown = curButton && ButtonPrev;
    ButtonPrev = curButton;
    
    if (ButtonReleased) {
      if (ButtonTimer < BUTTON_LONG) {
        // Short button press registered when button released prior to threshold
        ButtonPressShort = true;
      }
    }
    
    if (ButtonDown) {
      if (ButtonTimer == BUTTON_LONG) {
        // Long button press registered as we cross the threshold with the button pressed
        ButtonPressLong = true;
        ButtonTimer++; // Push timer past threshold to prevent it from counting anymore
      } else {
        if (ButtonTimer < BUTTON_LONG) {
          ButtonTimer++;
        }
      }
    } else {
      ButtonTimer = 0;
    }

    // Sample and update our average potentiometer array
    unsigned int Temp;
    Temp = analogRead(AN_POT);
    PotAvgBuff[AvgBuffPtr] = Temp;
    AvgBuffPtr = (AvgBuffPtr + 1) & 0x03;
  }
}


unsigned int CalcAvgPotVal() {
  unsigned int RetVal = 0;
  
  for (byte i=0; i<4; i++) {
    RetVal += PotAvgBuff[i];
  }
  return (RetVal >> 2);
}


byte GetPotDigit() {
  unsigned int CurPotVal;
  byte i;
  
  CurPotVal = CalcAvgPotVal();
  for (i=0; i<10; i++) {
    if (CurPotVal < ((i+1)*POT_N + (i*POT_M))) {
      return i;
    }
  }
  return 9;  // special case for few counts above final computed
             // value due to rounding errors in M and N
}
