void SetupProg1() {
  OPMode = OP_MODE_PROG_1;
  
  // Set a bogus previous pot value to force the first evaluation to set the current color
  PrevPotVal = 1024;
}


void EvalOpProg1() {
  // Prog1 uses the potentiometer to face through a rainbow of color:
  // Black->White->Red->Yellow->Green->Cyan->Blue->Magenta->Red
  //
  // Check for a change in the pot value
  unsigned int CurPotVal;
  CurPotVal = CalcAvgPotVal();
  if (CurPotVal != PrevPotVal) {
    CalcIndexedRGB(CurPotVal);
    SetPWM(CurRed, CurGreen, CurBlue);
    PrevPotVal = CurPotVal;
  }
}


void EvalUiProg1() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 2
    UIMode = UI_MODE_PROG_2;
    SetupProg2();
    SetupProg2Display();
  }
}


void CalcIndexedRGB(unsigned int index) {
 // Calculate current color values based on octant
 // Index Values      Color               Actions
 // ----------------------------------------------------
 //    0 -  127       Black -> White      RGB   0 -> 255
 //  128 -  255       White -> Red        GB  255 ->   0
 //  256 -  383       Red -> Yellow       G     0 -> 255
 //  384 -  511       Yellow -> Green     R   255 ->   0
 //  512 -  639       Green -> Cyan       B     0 -> 255
 //  640 -  767       Cyan -> Blue        G   255 ->   0
 //  768 -  895       Blue -> Magenta     R     0 -> 255
 //  896 - 1023       Magenta -> Red      B   255 ->   0
 //
 byte IndexMsb = (index >> 7) & 0x7;  // IndexMsb = index[9:7]
 byte IndexLsb = (index & 0x7F) << 1; // IndexLsb = index[6:0] << 1
 switch (IndexMsb) {
    case 0: //    0 -  127       Black -> White      RGB   0 -> 255
      CurRed = IndexLsb;
      CurGreen = IndexLsb;
      CurBlue = IndexLsb;
      break;
        
    case 1: //  128 -  255       White -> Red        GB  255 ->   0
      CurRed = 255;
      CurGreen = 254 - IndexLsb;
      CurBlue = CurGreen;
      break;
        
    case 2: //  256 -  383       Red -> Yellow       G     0 -> 255
      CurRed = 255;
      CurGreen = IndexLsb;
      CurBlue = 0;
      break;
        
    case 3: //  384 -  511       Yellow -> Green     R   255 ->   0
      CurRed = 254 - IndexLsb;
      CurGreen = 255;
      CurBlue = 0;
      break;
        
    case 4: //  512 -  639       Green -> Cyan       B     0 -> 255
      CurRed = 0;
      CurGreen = 255;
      CurBlue = IndexLsb;
      break;
        
    case 5: //  640 -  767       Cyan -> Blue        G   255 ->   0
      CurRed = 0;
      CurGreen = 254 - IndexLsb;
      CurBlue = 255;
      break;
        
    case 6: //  768 -  895       Blue -> Magenta     R     0 -> 255
      CurRed = IndexLsb;
      CurGreen = 0;
      CurBlue = 255;
      break;
        
    case 7: //  896 - 1023       Magenta -> Red      B   255 ->   0
      CurRed = 255;
      CurGreen = 0;
      CurBlue = 254 - IndexLsb;
  }
}


void SetupProg2() {
  OPMode = OP_MODE_PROG_2;
  
  // Turn off the LEDs
  SetPWM(0, 0, 0);
  
  // Get the initial strobe speed
  ProgDelay = GetStrobeDelay();
}


byte GetStrobeDelay() {
  byte fps;
  
  if (UIMode == UI_MODE_DMX) {
    // Calculate the rate based on channel 0 offset from start of strobe range: 1-33 flashes/sec
    fps = ((dmxvalue[0] - 64) >> 1) + 1;  // Offset/2 + 1
  } else { // UI_MODE_PROG_2
    // Calculate the rate based on the potentiometer value: 1-33 flashes/sec
    fps = (CalcAvgPotVal() >> 5) + 1;     // Avg Pot Val/32 + 1
  }
  return (1000 / fps / 5);
}


void EvalOpProg2() {
  unsigned long PrevFlashTime;
  unsigned long CurFlashTime;
  unsigned long deltaT = 0;
  
  // Prog2 uses the potentiometer or channel 0 offset to select a strobe speed using
  // the CurRed/Green/Blue values.
  //
  // Note: Flash time should be at least one complete PWM cycle (2040 uSec)
  if (--ProgDelay == 0) {
    // Get the current timer value
    PrevFlashTime = micros();
    CurFlashTime = PrevFlashTime;

    // Start the strobe
    SetPWM(CurRed, CurGreen, CurBlue);
    
    // Wait until the strobe period expires
    while (deltaT < 2040) {
      CurFlashTime = micros();
      if (CurFlashTime < PrevFlashTime) {
        deltaT = PrevFlashTime - CurFlashTime;
      } else {
        deltaT = CurFlashTime - PrevFlashTime;
      }
    }
    
    // End the strobe
    SetPWM(0, 0, 0);
    ProgDelay = GetStrobeDelay();
  }
}


void EvalUiProg2() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 3
    UIMode = UI_MODE_PROG_3;
    SetupProg3();
    SetupProg3Display();
  }
}


void SetupProg3() {
  OPMode = OP_MODE_PROG_3;
  ProgIndex = 256; // Start with red
  ProgDelay = GetFadeDelay();
}


byte GetFadeDelay() {
  byte nEvalIntervals;
  
  if (UIMode == UI_MODE_DMX) {
    // Calculate the change delay based on channel 0 offset from start of range
    // We guard-band the offset
    //   0-1 => 20 eval intervals per change (10 changes/sec)
    //   2-61 => 20-1 eval intervals per change
    //   62-63 => 1 eval intervals per change (200 changes/sec)
    //
    if (OPMode == OP_MODE_PROG_3) {
      nEvalIntervals = dmxvalue[0] - 128;
    } else {
      nEvalIntervals = dmxvalue[0] - 192;
    }
    
    if (nEvalIntervals < 2) {
      nEvalIntervals = 20;
    } else if (nEvalIntervals > 61) {
      nEvalIntervals = 1;
    } else {
      nEvalIntervals = 20 - (nEvalIntervals / 3);
    }
  } else { // UI_MODE_PROG_2
    // Calculate the rate based on the potentiometer value
    // Dividing the pot value by 51 results in a range of approximately 1-21 eval
    // intervals per change
    nEvalIntervals = 21 - (CalcAvgPotVal() / 51);
  }
  return (nEvalIntervals);
}


void EvalOpProg3() {
  // Sequential Fade: Red->Yellow->Green->Cyan->Blue->Magenta->Red
  // 
  if (--ProgDelay == 0) {
    CalcIndexedRGB(ProgIndex);
    if (++ProgIndex == 1024) ProgIndex = 256;
    SetPWM(CurRed, CurGreen, CurBlue);
    ProgDelay = GetFadeDelay();
  }
}


void EvalUiProg3() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 4
    UIMode = UI_MODE_PROG_4;
    SetupProg4();
    SetupProg4Display();
  }
}


void SetupProg4() {
  OPMode = OP_MODE_PROG_4;
  SelectRandomColor();           // Setup 1st randomly selected color
  SetupInterpolator();
  ProgDelay = GetFadeDelay();  
}


// SelectRandomColor selects biased colors to ensure higher saturation.
// It does that by picking one intense color, a second less intense color
// and a third even less intense color then randomly assigning them to
// R, G and B.
void SelectRandomColor() {
  unsigned long NewColor[3];
  unsigned long ColorOrder;
  
  // Select the colors
  NewColor[0] = random(0xA0, 0x100);
  NewColor[1] = random(0x20, 0xB0);
  NewColor[2] = random(0x00, 0x18);
  
  // Select the assignment
  ColorOrder = random(0,6);
  switch (ColorOrder) {
    case 0: // RGB
      NextRed = NewColor[0];
      NextGreen = NewColor[1];
      NextBlue = NewColor[2];
      break;
      
    case 1: // RBG
      NextRed = NewColor[0];
      NextGreen = NewColor[2];
      NextBlue = NewColor[1];
      break;

    case 2: // GRB
      NextRed = NewColor[1];
      NextGreen = NewColor[0];
      NextBlue = NewColor[2];
      break;

    case 3: // GBR
      NextRed = NewColor[2];
      NextGreen = NewColor[0];
      NextBlue = NewColor[1];
      break;

    case 4: // BRG
      NextRed = NewColor[1];
      NextGreen = NewColor[2];
      NextBlue = NewColor[0];
      break;

    case 5: // BGR
      NextRed = NewColor[2];
      NextGreen = NewColor[1];
      NextBlue = NewColor[0];
      break;
  }
}


void SetupInterpolator() {
  int Temp;  // For the abs function (we're not supposed to include function calls in it)
  
  // Compute the delta and sign values
  Temp = (int) NextRed - (int) CurRed;
  DeltaVal[0] = abs(Temp);
  InterpSign[0] = (NextRed < CurRed);
  
  Temp = (int) NextGreen - (int) CurGreen;
  DeltaVal[1] = abs(Temp);
  InterpSign[1] = (NextGreen < CurGreen);
  
  Temp = (int) NextBlue - (int) CurBlue;
  DeltaVal[2] = abs(Temp);
  InterpSign[2] = (NextBlue < CurBlue);
  
  // Determine the major axis (will change by 1 per evaluation)
  // and the increment values for each axis (the 2 minor axis will be fractional)
  if ((DeltaVal[0] > DeltaVal[1]) && (DeltaVal[0] > DeltaVal[2])) {
    // Red is major
    IncVal[0] = 0x010000;
    IncVal[1] = (DeltaVal[1] << 16) / DeltaVal[0];
    IncVal[2] = (DeltaVal[2] << 16) / DeltaVal[0];
    Major = 0;
  } else if ((DeltaVal[1] > DeltaVal[0]) && (DeltaVal[1] > DeltaVal[2])) {
    // Green is major
    IncVal[0] = (DeltaVal[0] << 16) / DeltaVal[1];
    IncVal[1] = 0x010000;
    IncVal[2] = (DeltaVal[2] << 16) / DeltaVal[1];
    Major = 1;
  } else {
    // Blue is major
    IncVal[0] = (DeltaVal[0] << 16) / DeltaVal[2];
    IncVal[1] = (DeltaVal[1] << 16) / DeltaVal[2];
    IncVal[2] = 0x010000;
    Major = 2;
  }
  
  // Setup our interpolator color values (format: {Color[7:0] "." 0x0000 }
  InterpColor[0] = (unsigned long) CurRed << 16;
  InterpColor[1] = (unsigned long) CurGreen << 16;
  InterpColor[2] = (unsigned long) CurBlue << 16;
}


byte RoundFrac(long FracNum) {
  byte TempByte = (FracNum >> 16) & 0xFF;
  
  if ((FracNum & 0xFFFF) >= 0x8000) {
    // Round up
    return (TempByte + 1);
  } else {
    // No rounding
    return (TempByte);
  }
}


void EvalOpProg4() {
  // Random Fade: From one CurRGB to NextRGB
  // 
  if (--ProgDelay == 0) {
    if (DeltaVal[Major] != 0) {
      // Evaluate this step
      DeltaVal[Major]--;
      
      // Calculate the new Cur values
      for (byte i=0; i<3; i++) {
        if (InterpSign[i]) {
          // Decreasing color value
          InterpColor[i] -= IncVal[i];
        } else {
          // Increasing color value
          InterpColor[i] += IncVal[i];
        }
      }
    
      // Update the color
      CurRed = RoundFrac(InterpColor[0]);
      CurGreen = RoundFrac(InterpColor[1]);
      CurBlue = RoundFrac(InterpColor[2]);
      SetPWM(CurRed, CurGreen, CurBlue);
    } else {
      // Setup for the next fade
      SelectRandomColor();
      SetupInterpolator();
    }
    ProgDelay = GetFadeDelay();
  }
}


void EvalUiProg4() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to DMX mode
    UIMode = UI_MODE_DMX;
    SetupDMX();
    SetupDmxDisplay();
  }
}
