void CheckEEPsanity() {
  byte MagicByte = EEPROM.read(EEP_MAGIC_ADDR);
  
  if (MagicByte != EEP_MAGIC_BYTE) {
    // Set up the initial EEPROM values
    EEPROM.write(EEP_MAGIC_ADDR, EEP_MAGIC_BYTE);
    
    // Setup the initial DMX address to 001
    EEPROM.write(EEP_DMX_H_ADDR, 0);
    EEPROM.write(EEP_DMX_L_ADDR, 1);
    
    // Setup the initial operating mode to DMX
    EEPROM.write(EEP_USER_MODE, UI_MODE_DMX);
    
    // Setup the initial color to 0 (black)
    EEPROM.write(EEP_C_H_ADDR, 0);
    EEPROM.write(EEP_C_L_ADDR, 0);
  }
}


unsigned int GetDmxAddress() {
  byte HighByte = EEPROM.read(EEP_DMX_H_ADDR);
  byte LowByte = EEPROM.read(EEP_DMX_L_ADDR);
  unsigned int Addr = (HighByte << 8) | LowByte;
  
  if (Addr == 0) {
    Addr = 1;
  } else if (Addr > MAX_DMX_ADDR) {
    Addr = MAX_DMX_ADDR;
  }
  
  return (Addr);
}


void StoreDmxAddress(unsigned int workingAddr) {
  unsigned int TrueAddress = workingAddr - 3;  // Adjust for the real address
  
  if (GetDmxAddress() != TrueAddress) {
    EEPROM.write(EEP_DMX_H_ADDR, (TrueAddress >> 8) & 0x01);
    EEPROM.write(EEP_DMX_L_ADDR, (TrueAddress & 0xFF));
  }
}


byte GetFixtureMode() {
  byte EepData = EEPROM.read(EEP_USER_MODE);
  
  if ((EepData != UI_MODE_DMX) && (EepData != UI_MODE_PROG_1) &&
      (EepData != UI_MODE_PROG_2) && (EepData != UI_MODE_PROG_3) &&
      (EepData != UI_MODE_PROG_4)) {
    // Illegal value
    EepData = UI_MODE_DMX;
  }
  
  return (EepData);
}


void StoreFixtureMode(byte nMode) {
  if (GetFixtureMode() != nMode) {
    EEPROM.write(EEP_USER_MODE, nMode);
  }
}


unsigned int GetFixtureIndexedColor() {
  byte HighByte = EEPROM.read(EEP_C_H_ADDR);
  byte LowByte = EEPROM.read(EEP_C_L_ADDR);
  unsigned int fColor = (HighByte << 8) | LowByte;
  
  return(fColor & 0x3FF);  // limit to 0-1023
}


void StoreFixtureIndexedColor(unsigned int cIndex) {
  byte HighByte = (cIndex >> 8) & 0x03;
  byte LowByte = cIndex & 0xFF;
  
  if (GetFixtureIndexedColor() != cIndex) {
    EEPROM.write(EEP_C_H_ADDR, HighByte);
    EEPROM.write(EEP_C_L_ADDR, LowByte);
  }
}
  

// Function to only update mode-related information when we've remained in a mode
// for longer than a minimum threshold (to reduce EEPROM writes when moving
// quickly through modes)
void EvalModeChange() {
  
  // Look to see if we need to start/re-start the timer
  if ((UIMode != PrevUIMode) && (UIMode != UI_MODE_SET_ADDR)) {
    // Start the timer
    ModeChangeTimer = MODE_CHNG_CNT + 1;
    
    PrevUIMode = UIMode;
  }
  
  // Evalute the timer to see if it's time to update
  if (ModeChangeTimer != 0) {
    if (--ModeChangeTimer == 0) {
      // Store the new mode
      StoreFixtureMode(UIMode);
      
      // Check if we need to store the current indexed color
      if (UIMode == UI_MODE_PROG_2) {
        // Store the color if we're in strobe mode
        StoreFixtureIndexedColor(PrevPotVal);
      }
    }
  }
}
